@@ -89,7 +89,7 @@ encode.sf <- function(obj, strip = FALSE, ...) {
   
   if(!strip) sfAttrs <- sfGeometryAttributes(obj)
 
-  obj[[geomCol]] <- lst
+  obj[[geomCol]] <- lst[['XY']]
   
   ## strip attributes
   obj <- structure(obj, sf_column = NULL, agr = NULL, class = setdiff(class(obj), "sf"))
@@ -97,20 +97,45 @@ encode.sf <- function(obj, strip = FALSE, ...) {
   attr(obj[[geomCol]], 'class') <- c('encoded_column', class(obj[[geomCol]]) )
   attr(obj, 'encoded_column') <- geomCol
   
-  if(!strip) {
+  ## TODO(remove this vapply step and return from rcpp a flag if the ZM attrs are attached)
+  if (any(vapply(lst[['ZM']], length, 0L)) > 0) {
+    zmCol <- paste0(geomCol, "ZM")
+    zmCol <- make.names(c(names(obj), zmCol), unique = T)
+    zmCol <- zmCol[length(zmCol)]
+    
+    obj[[zmCol]] = lst[['ZM']]
+    
+    ## attach ZM attribute column
+    attr(obj, 'zm_column') <- zmCol
+    attr(obj[[zmCol]], 'class') <- c('zm_column', class(obj[[zmCol]]))
+  }
+  
+  if (!strip) {
     attr(obj, "sfAttributes") <- sfAttrs
     
     if(!inherits(obj, 'sfencoded'))
       attr(obj, 'class') <- c("sfencoded", attr(obj, 'class'))
-  }else{
+  } else {
     if(!inherits(obj, 'sfencodedLite'))
       attr(obj, 'class') <- c("sfencodedLite", attr(obj, 'class'))
   }
+  
   return(obj)
 }
 
 #' @export
-encode.sfc <- function(obj, strip = FALSE, ...) rcpp_encodeSfGeometry(obj, strip)
+encode.sfc <- function(obj, strip = FALSE, ...) {
+  lst <- rcpp_encodeSfGeometry(obj, strip)
+  
+  ## TODO(remove this vapply step and return from rcpp a flag if the ZM attrs are attached)
+  if (all(vapply(lst[['ZM']], length, 0L)) == 0) {
+    
+    lst[['ZM']] <- NULL
+    lst <- lst[['XY']] ## to keep previous behaviour? 
+  }
+  
+  return( lst )
+}
 
 #' @rdname encode
 #' @param lon vector of longitudes
