@@ -21,21 +21,31 @@ str.encoded_column <- strSfEncoded
 #' @export
 str.wkt_column <- strSfEncoded
 
+#' @export
+str.zm_column <- strSfEncoded
+
 
 #' @export
 `[.sfencoded` <- function(x, i, j, ..., drop = TRUE) {
-
+  
+  encodedClass <- attr(x, 'class')[1]
   geomColumn <- attr(x, "encoded_column")
   wktColumn <- attr(x, "wkt_column")
+  zmColumn <- attr(x, "zm_column")
   attr(x, "sfAttributes") <- NULL
 
   x <- NextMethod()
   x <- attachEncodedAttribute(x, geomColumn, "encoded_column")
   x <- attachEncodedAttribute(x, wktColumn, "wkt_column")
+  x <- attachEncodedAttribute(x, zmColumn, "zm_column")
 
   if( is.null(attr(x, "encoded_column")) && is.null(attr(x, "wkt_column")) ){
     x <- removeSfencodedClass(x)
+  } else {
+    ## re-attach 'sfencoded' attribute?
+    attr(x, 'class') <- unique(c(encodedClass, attr(x, 'class')))
   }
+  
   return(x)
 }
 
@@ -62,6 +72,7 @@ removeSfEncodedAttributes <- function(x) {
 
   geomCol <- attr(x, "encoded_column")
   wktCol <- attr(x, "wkt_column")
+  zmCol <- attr(x, "zm_column")
   
   if(!is.null(geomCol) && geomCol %in% names(x)) {
     x[[geomCol]] <- sapply(x[[geomCol]], function(y) { 
@@ -77,11 +88,20 @@ removeSfEncodedAttributes <- function(x) {
     attr(x[[wktCol]], "class") <- NULL
   }
   
+  if(!is.null(zmCol) && zmCol %in% names(x)) {
+    x[[zmCol]] <- sapply(x[[zmCol]], function(y) { 
+      attr(y, "zm") <- NULL 
+      return(y) 
+    })
+    
+    attr(x[[zmCol]], "class") <- NULL
+  }
+  
   attr(x, "encoded_column") <- NULL
   attr(x, "wkt_column") <- NULL
+  attr(x, "zm_column") <- NULL
   attr(x, "sfAttributes") <- NULL
   
-  # print("-- attributes removed --")
   return(x)
 }
 
@@ -107,6 +127,7 @@ printSfEncoded <- function(x, ...) {
   
   encoded <- attr(x, "encoded_column")
   wkt <- attr(x, "wkt_column")
+  zm <- attr(x, 'zm_column')
 
   if(!is.null(encoded)) {
     e <- x[[encoded]]
@@ -122,12 +143,29 @@ printSfEncoded <- function(x, ...) {
     x[, wkt] <- w
   }
   
+  if(!is.null(zm) ) {
+    z <- x[[zm]]
+    z <- printZMattributes(z)
+    z <- stats::setNames(data.frame(z), zm)
+    x[, zm] <- z
+  }
+  
   x <- removeSfencodedClass(x)
   
   print(x)
   invisible(x)
 }
 
+printZMattributes <- function(zm) {
+  z <- vapply(zm, function(x) {
+    paste0(
+      substr(x[1], 1, pmin(nchar(x[1]), 20))
+      , "..."
+    )
+  }, "" )
+  return(z)
+}
+
 printSfEncodedPrefix <- function(e, encType) {
   
   if(encType == "sfencoded") {
@@ -138,7 +176,7 @@ printSfEncodedPrefix <- function(e, encType) {
         "..."
       )
     }, "" )
-  }else{
+  } else {
     e <- vapply(e, function(z) {
       paste0(
         substr(z[1], 1, pmin(nchar(z[1]), 20)),
@@ -147,6 +185,5 @@ printSfEncodedPrefix <- function(e, encType) {
     }, "" )
   }
   return(e)
-  
 }
 
