@@ -18,6 +18,16 @@ test_that("sfencoded converted to data.frame",  {
   expect_true(class(x) == "data.frame")
 })
 
+test_that("subsetting retains all class attributes", {
+  
+  testthat::skip_on_cran()
+  library(sf)
+  nc <- sf::st_read(system.file("shape/nc.shp", package="sf"), quiet = T)
+  enc <- encode(nc)
+  expect_true(all(attr(enc, 'class') == attr(enc[1, ], 'class')))
+
+})
+
 test_that("sfencoded attributes are removed", {
   
   testthat::skip_on_cran()
@@ -35,6 +45,28 @@ test_that("sfencoded attributes are removed", {
   expect_true(sum(c("encoded_column", "sfAttributes") %in% names(attributes(x))) == 0)
 })
 
+test_that("zm attributes are removed", {
+  
+  testthat::skip_on_cran()
+  library(sf)
+  
+  z <- 1:21
+  zm <- 1:36
+  pz <- sf::st_point(c(1,2,3))
+  pzm <- sf::st_point(1:4)
+  sf1 <- sf::st_sf(geometry = sf::st_sfc(pz))
+  sf2 <- sf::st_sf(geometry = sf::st_sfc(pzm))
+  sf <- rbind(sf1, sf2)
+  enc <- encode(sf)
+  
+  x <- googlePolylines:::removeSfEncodedAttributes(enc)
+  expect_true(sum(c("zm_column", "sfAttributes") %in% names(attributes(x))) == 0)
+  
+  df <- as.data.frame(x)
+  expect_null(attributes(df[, 'geometryZM']))
+  
+})
+
 test_that("correct structure is printed", {
   
   testthat::skip_on_cran()
@@ -54,7 +86,7 @@ test_that("encoded attribute is attached", {
   expect_true(attr(df, 'encoded_column') == 'polyline')
 })
 
-test_that("encoed objects printed", {
+test_that("encoded objects printed", {
   
   enc <- data.frame(polyline = "abc", stringsAsFactors = F)
   attr(enc, 'class') <- c('sfencoded', class(enc))
@@ -107,6 +139,17 @@ test_that("subsetting rows and columns works", {
   expect_true(ncol(encl[1:5, ]) == ncol(enc))
   expect_true(ncol(encl[1:5, c("AREA","PERIMETER")]) == 2)
   expect_true(ncol(as.data.frame(encl[1:5, c("AREA","PERIMETER")])) == 2)
+  
+  ## elevation (zm) attributes
+  sf <- sf::st_sf(geometry = sf::st_sfc(sf::st_point(1:3)))
+  sf$id <- 1
+  enc <- encode( sf )
+  
+  ## subsetting removing the elevation
+  expect_true(!"geometryZM" %in% names(enc[1, c("geometry", "id")]))
+  ## subsetting keeping the elvation
+  expect_true(all(c("id", "geometryZM") %in% names(enc[1, c("id", "geometryZM")])))
+  
 })
 
 
